import finstats.stats as stats

if __name__ == '__main__':
  # stats.cal_alpha_beta('sh600519', 'sh000300', datalen=1000)
  # stats.cal_alpha_beta('sz300750', 'sh000300', datalen=1000)
  # stats.cal_alpha_beta('sh510300', 'sh000300', datalen=1000)
  # stats.cal_alpha_beta('sh510050', 'sh000300', datalen=1000)
  stats.finstats()